require "import"
import "android.app.*"
import "android.content.*"
import "android.content.pm.*"
import "java.io.*"
import "android.net.*"
import "android.os.Environment"
import "android.os.Vibrator"
import "android.speech.RecognizerIntent"
import "android.speech.tts.TextToSpeech"

local Locale=luajava.bindClass("java.util.Locale")
local pm=activity.getPackageManager()
local vibe=activity.getSystemService(Context.VIBRATOR_SERVICE)
local tts
local apps={}

function initTTS()
  tts=TextToSpeech(activity,TextToSpeech.OnInitListener{
    onInit=function(status)
      if status==TextToSpeech.SUCCESS then
        tts.setLanguage(Locale("ro"))
      end
    end
  })
end

function speakText(text)
  if tts then
    tts.speak(text,TextToSpeech.QUEUE_FLUSH,nil,"ttsID")
  end
end

local allApps=pm.getInstalledApplications(PackageManager.GET_META_DATA)
for i=0,allApps.size()-1 do
  local info=allApps.get(i)
  local label=info.loadLabel(pm)
  local pkg=info.packageName
  if pm.getLaunchIntentForPackage(pkg) then
    table.insert(apps,{label=tostring(label),packageName=pkg})
  end
end
table.sort(apps,function(a,b)return a.label:lower()<b.label:lower()end)

local function shareApkWhatsApp(pkg)
  local pi=pm.getPackageInfo(pkg,PackageManager.GET_META_DATA)
  if pi and pi.applicationInfo and pi.applicationInfo.sourceDir then
    local apkPath=pi.applicationInfo.sourceDir
    local appName=tostring(pm.getApplicationLabel(pi.applicationInfo))
    local ver=tostring(pi.versionName)
    local fileName=appName.."_v"..ver..".apk"
    local externalDir=File(Environment.getExternalStoragePublicDirectory(Environment.DIRECTORY_DOWNLOADS),"shared_apk_whatsapp")
    externalDir.mkdirs()
    local sharedFile=File(externalDir,fileName)
    LuaUtil.copyFile(apkPath,sharedFile.getAbsolutePath())
    speakText("Partajează APK-ul "..appName.." versiunea "..ver..". Selectați un contact.")
    local sendIntent=Intent()
    sendIntent.setAction(Intent.ACTION_SEND)
    sendIntent.setPackage("com.whatsapp")
    sendIntent.setType("application/vnd.android.package-archive")
    sendIntent.putExtra(Intent.EXTRA_STREAM,activity.getUriForFile(sharedFile))
    sendIntent.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION)
    pcall(function() activity.startActivity(sendIntent) end)
  end
end

function onCreate()
  initTTS()
  vibe.vibrate(150)
  startVoiceRecognition()
end

function startVoiceRecognition()
  local intent=Intent(RecognizerIntent.ACTION_RECOGNIZE_SPEECH)
  intent.putExtra(RecognizerIntent.EXTRA_LANGUAGE_MODEL,RecognizerIntent.LANGUAGE_MODEL_FREE_FORM)
  intent.putExtra(RecognizerIntent.EXTRA_PROMPT,"Spune numele aplicației...")
  activity.startActivityForResult(intent,1)
end

function onActivityResult(requestCode,resultCode,data)
  if requestCode==1 and resultCode==-1 and data then
    local matches=data.getStringArrayListExtra(RecognizerIntent.EXTRA_RESULTS)
    if matches and matches.size()>0 then
      local spoken=matches.get(0)
      local found=false
      for i=1,#apps do
        if apps[i].label:lower():find(spoken:lower()) then
          shareApkWhatsApp(apps[i].packageName)
          found=true
          break
        end
      end
      if not found then
        speakText("Am înțeles: "..spoken..", dar nu am găsit aplicația în listă.")
      end
    end
  end
end